' === temp9700test.bas =======================================
' This program measures temperature using an MCP9700A sensor.
' It's essentially the same as the Temp9700A.bas program that
' we discussed in the December 2012 PICAXE Primer Column.
' The only difference is, this time, we are displaying all
' three results (ADCval, tempC, and tempF) in the terminal.

' === Directives ===
  #com 4                         'specify serial port
  #picaxe 08M2                   'specify processor
  #terminal 9600                 'open the terminal window
  #no_data                       'save time downloading
  
' === Constants ====
  symbol LED = C.0               'debugging LED on pin C.0
  symbol MCP = C.4               'MCP9700A on pin C.4

' === Variables =====
  symbol  ADCval = w0            'stores raw temperature data
  symbol   tempC = w1            'stores Celsius temp (x10)
  symbol   tempF = w2            'stores Fahrenheit temp (x10)
  symbol degrees = b6            'stores integer value of temp
  symbol  tenths = b7            'stores tenths value of temp
  symbol   signC = b8            'stores sign of tempC
  symbol   signF = b9            'stores sign of tempF
  
' === Begin Main Program =====================================

  fvrsetup FVR2048                     'set FVR at 2048mV
  adcconfig %011                       'Vref+=FVR, Vref-=0V
  setfreq m8
  
  pause 100                            'for 9700 startup
    
do
  high LED                             'blink debugging LED
  wait 1                     
  low LED
  
  readadc10 MCP,ADCval                 'fetch raw data
  sertxd ("ADCval = ",#ADCval,"; ")    'display it in terminal
      
  gosub convertRaw2tempC  
  degrees = tempC / 10
  tenths  = tempC //10
  
  ' display tempC in terminal
  sertxd ("temp is ",#degrees,".",#tenths," C,")
 
  gosub convertC2F
  degrees = tempF / 10
  tenths  = tempF //10 
  if tenths >= 5 then
    inc degrees
  endif
  
  ' display tempF in terminal
  sertxd (" or ",#degrees,".",#tenths," F",CR,LF)
   
  wait 9                               'slow down a bit
loop

' === End Main Program: Subroutines Follow ===================

convertRaw2tempC:
  if ADCval < 250 then                 'test for sign
    tempC = 2 * ADCval
    tempC = 500-ADCval
    signC = "-"
  else
    tempC = 2 * ADCval - 500
    signC = "+"
  endif
  return
  
convertC2F:
  select case ADCval
    case 0 to 160                      'C=-50 to -17.7
      tempF = tempC *9 / 5 - 320       '(F negative)
      signF = "-"
    case 161 to 249                    'C=-17.8 to -0.1
      tempF = tempC *9 / 5             '(F positive)
      tempF = 320 - tempF
      signF = "+" 
    case 250 to 870                    'C= 0 to +125
      tempF = tempC *9 / 5 + 320       '(F=+32 to +255)
      signF = "+"
  endselect 
  return